from django.db import models
from jsonfield import JSONField
#from django.contrib.postgres.fields import ArrayField
#from reqman.apps.common.models import CoreModel
from reqman.apps.account.models import User
from reqman.apps.reqtool.models.main_req import * # create_main, ITEM_CHOICES, STATE_VALUE_CHOICES, STATE_CHOICES, SYSTEM_CHOICES, FUNCTION_CHOICES, FLOW_CHOICES
from reqman.apps.reqtool.models.suffix_req import * #create_suffix
from reqman.apps.reqtool.rest_api.services.parse_ontologies import *

from pygments.formatters.html import HtmlFormatter
from pygments import highlight

#get prefix syntax from the Onotlogy#
#print(get_prefix_new_syntax())
PREFIX_SYNTAX = get_prefix_syntax()


SIMPLE_PREFIX_CHOICES = (   ("",""),  
                            ("P1", "If/Unless <logical expression>"), 
                            ("P2", "As soon as <occuring functionality>"), 
                            ("P3", "As long as <occuring functionality>")
                        )

LOGIC_CONNECTIVE = (    ("or","OR"), 
                        ("and", "AND"), 
                        ("xor", "XOR")
                    )

LOGICAL_EXPRESSION = (  ("",""), 
                        ("state value constraint", "state value constraint"), 
                        ("occuring functionality", "occuring functionality")
                    )


VERB_CHOICES_PREFIX = get_prefix_verbs()


#Must read this info from the Ontology
PREFIX_CHOICES = (   ("If", "If"), ("Unless", "Unless"),
                            ("As soon as", "As soon as"), 
                            ("As long as", "As long as"), ("Once", "Once")
                        )


def create_prefixgroup(instance):
    '''Create the PrefixGroup Model for the Boilerplate'''
    try :
        prefix = PrefixGroup.objects.get(boilerplate = instance, prefix_owner = instance.owner)
        if instance.has_prefix:
            return
        prefix = PrefixGroup.objects.get(boilerplate = instance, prefix_owner = instance.owner)
        prefix.delete()
    except PrefixGroup.DoesNotExist:
        if instance.has_prefix:
            prefix = PrefixGroup(boilerplate = instance, prefix_owner = instance.owner)
            prefix.save()


def create_prefix(instance):
        prefix = Prefix(prefix_boilerplate = instance.boilerplate, prefixgroup = instance)
        prefix.save()


def create_data(instance):
        data = BoilerplateData(boilerplate_data_id = instance, owner_data = instance.owner, title_data= instance.title, group_of_boilerplate_data = instance.group_of_boilerplate )
        data.save()


def create_extra_prefix(instance, now_prefix):
    prefix = Prefix(prefixgroup = instance.prefixgroup, prefix_boilerplate = instance.prefix_boilerplate)
    prefix.save()


def create_infer_result_and_classes_instances(instance):
    infer = InferenceResults(owner_infer= instance.boilerplate_owner, infer_group_of_boilerplate = instance)
    infer.save()
    a = (("If", "If"), ("Unless", "Unless"),("As soon as", "As soon as"), ("As long as", "As long as"), ("Once", "Once"))
    jsonObj = json.dumps(a)
    classes_inctances = BoilerplateGroupClassesInstances ( classes_instances_owner= instance.boilerplate_owner, classes_instances_group_of_boilerplate = instance,
    prefix_choices = jsonObj, prefix_logcon_choices =json.dumps(LOGIC_CONNECTIVE), prefix_verb_choices =json.dumps(STATE_CHOICES + VERB_CHOICES_PREFIX),
    main_class_choices =json.dumps(MAIN_CHOICES), system_choices = json.dumps(SYSTEM_CHOICES), function_choices =json.dumps(FUNCTION_CHOICES),
    quantifier_choices=json.dumps(QUANTIFIER_CHOICES), number_unit_choices=json.dumps(NUMBER_UNITS_CHOICES), item_choices=json.dumps(ITEM_CHOICES),
    inteface_choices=json.dumps(INTERFACE_CHOICES), connection_choices=json.dumps(CONNECTION_CHOICES), flow_choices=json.dumps(FLOW_CHOICES),
    state_choices=json.dumps(STATE_CHOICES),state_set_choices=json.dumps(STATE_SET_CHOICES), state_value_choices=json.dumps(STATE_VALUE_CHOICES),
    main_verb_choices=json.dumps(VERB_CHOICES), numerical_affirmative_choices=json.dumps(NUMERICAL_AFFIRMATIVE_CHOICES),
    suffix_choices=json.dumps(S_CHOICES) )
    classes_inctances.save()





class BoilerplateGroup(models.Model):
    boilerplate_owner = models.ForeignKey(User, related_name='boilerplate_group_owner', on_delete=models.CASCADE)
    title_bgroup = models.CharField(max_length=100, unique=True)
    description = models.CharField(max_length=500)

    def save(self, **kwargs):
        super(BoilerplateGroup, self).save(**kwargs)
        create_infer_result_and_classes_instances(self)


class Boilerplate(models.Model):
    owner = models.ForeignKey(User, related_name='boilerplate', on_delete=models.CASCADE)
    group_of_boilerplate = models.ForeignKey(BoilerplateGroup, related_name='owner_of_boilerplate', on_delete=models.CASCADE)
    created = models.DateTimeField(auto_now_add=True)
    title = models.CharField(max_length=30, unique=True)
    has_prefix = models.BooleanField(default=False)
    has_main = models.BooleanField(default=False)
    has_suffix = models.BooleanField(default=False)


    def __init__(self, *args, **kwargs):
            super(Boilerplate, self).__init__(*args, **kwargs)
            self.__important_fields = ['has_prefix', 'has_main', 'has_suffix']
            for field in self.__important_fields:
                setattr(self, '__original_%s' % field, getattr(self, field))
        

    def save(self, **kwargs):
        """Create prefix, main or suffix """
        super(Boilerplate, self).save(**kwargs)
        prev_prefix_list = []
        for field in self.__important_fields:
            orig = '__original_%s' % field
            previous_simple_prefix = getattr(self, orig)
            prev_prefix_list.append(previous_simple_prefix)
        create_prefixgroup(self)
        create_main(self)
        create_suffix(self)
        if (prev_prefix_list[0] == True and self.has_prefix == False ):
            pr = ""
            BoilerplateData.objects.filter(owner_data=self.owner, boilerplate_data_id = self).update(prefix_data = pr )
        elif (prev_prefix_list[1] == True and self.has_main == False ):
            pr = ""
            BoilerplateData.objects.filter(owner_data=self.owner, boilerplate_data_id = self).update(main_data = pr )
        elif (prev_prefix_list[2] == True and self.has_suffix == False ):
            pr = ""
            BoilerplateData.objects.filter(owner_data=self.owner, boilerplate_data_id = self).update(suffix_data = pr )
        elif(BoilerplateData.objects.filter(owner_data=self.owner, boilerplate_data_id = self).count() != 1):
            create_data(self)

        
    class Meta:
        ordering = ['created']


class PrefixGroup(models.Model):
    boilerplate = models.ForeignKey(Boilerplate, related_name='prefixgroup', on_delete=models.CASCADE)
    prefix_owner = models.ForeignKey(User, related_name='prefix_group_owner', on_delete=models.CASCADE)

    def save(self, **kwargs):
        super(PrefixGroup, self).save(**kwargs)
        create_prefix(self)
        


class Prefix(models.Model):
    #boilerplate = models.OneToOneField(Boilerplate, related_name='prefix', on_delete=models.CASCADE, primary_key=True)
    prefixgroup = models.ForeignKey(PrefixGroup, related_name='prefix', on_delete=models.CASCADE)
    prefix_boilerplate = models.ForeignKey(Boilerplate, related_name='prefix_boilerplate', on_delete=models.CASCADE)
    prefix = models.CharField(choices=PREFIX_CHOICES, max_length=100)
    #state value constraint
    system_fun_item = models.CharField(choices= (ITEM_CHOICES + SYSTEM_CHOICES + FUNCTION_CHOICES), max_length=100)
    state_or_verb =  models.CharField(choices=STATE_CHOICES + VERB_CHOICES_PREFIX, max_length=100)
    #state_value = models.CharField(choices=STATE_VALUE_CHOICES, max_length=100, blank=True)
    #occuring functionality
    #sys_fun = models.CharField(choices=, max_length=100, blank=True)
    #verb = models.CharField(choices=VERB_CHOICES, max_length=100, blank=True)
    item_function_flow_statevalue = models.CharField(choices=ITEM_CHOICES + FLOW_CHOICES + FUNCTION_CHOICES + STATE_VALUE_CHOICES, max_length=100)
    #Prefix value P1/P2/P3
    logical_expression =  models.CharField(choices=LOGICAL_EXPRESSION, max_length=100, blank=True)
    #Info for prefix
    logic_connective = models.CharField(choices=LOGIC_CONNECTIVE, default='', max_length=100, blank=True)
    simple_prefix = models.CharField(choices=SIMPLE_PREFIX_CHOICES, max_length=100)


    def __init__(self, *args, **kwargs):
        super(Prefix, self).__init__(*args, **kwargs)
        self.__important_fields = ['logic_connective']
        for field in self.__important_fields:
            setattr(self, '__original_%s' % field, getattr(self, field))


    def save(self, **kwargs):
        """Create P1, P2 or P3 """
        prev_prefix_list = []
        super(Prefix, self).save(**kwargs)
        for field in self.__important_fields:
            orig = '__original_%s' % field
            previous_simple_prefix = getattr(self, orig)
            prev_prefix_list.append(previous_simple_prefix)
        if previous_simple_prefix == "" and not self.logic_connective == "" :
            create_extra_prefix ( self, self.logic_connective)
        

    class Meta:
        ordering = ['prefix_boilerplate']


class BoilerplateData(models.Model):
    owner_data = models.ForeignKey(User, related_name='boilerplate_data', on_delete=models.CASCADE)
    group_of_boilerplate_data = models.ForeignKey(BoilerplateGroup, related_name='group_of_boilerplate_data', on_delete=models.CASCADE)    
    boilerplate_data_id = models.ForeignKey(Boilerplate, related_name='boilerplate_data_id', on_delete=models.CASCADE)
    title_data = models.CharField(default="", max_length=30, unique=True)
    prefix_data = models.CharField(default="", max_length=100, blank=True)
    main_data = models.CharField(default="", max_length=100, blank=True)
    suffix_data =  models.CharField(max_length=100, blank=True)

    class Meta:
        ordering = ['boilerplate_data_id']


class InferenceResults(models.Model):
    owner_infer = models.ForeignKey(User, related_name='owner_infer', on_delete=models.CASCADE)    
    infer_group_of_boilerplate = models.ForeignKey(BoilerplateGroup, related_name='infer_owner_of_boilerplate', on_delete=models.CASCADE)
    inference_data = models.CharField(default="", max_length=1000000, blank=True)

    class Meta:
        ordering = ['owner_infer']

import collections

class BoilerplateGroupClassesInstances(models.Model):
    classes_instances_group_of_boilerplate = models.ForeignKey(BoilerplateGroup, related_name='classes_instances_owner_of_boilerplate', on_delete=models.CASCADE)
    classes_instances_owner = models.ForeignKey(User, related_name='classes_instances_owner', on_delete=models.CASCADE)
    ontology_file = models.CharField(default="", max_length=100, blank=True)
    prefix_choices = JSONField()
    prefix_logcon_choices = JSONField()
    prefix_verb_choices = JSONField()
    main_class_choices =JSONField()
    system_choices=JSONField()
    function_choices = JSONField()
    quantifier_choices = JSONField()
    item_choices = JSONField()
    inteface_choices = JSONField()
    connection_choices = JSONField()
    flow_choices = JSONField()
    state_choices =JSONField()
    state_set_choices = JSONField()
    state_value_choices = JSONField()
    main_verb_choices = JSONField()

    numerical_affirmative_choices = JSONField()
    number_unit_choices =JSONField()

    suffix_choices = JSONField()

import json

def get_prefix_choices():
    #Must read this info from the Ontology
    a = (("If", "If"), ("Unless", "Unless"),("As soon as", "As soon as"), ("As long as", "As long as"), ("Once", "Once"))
    jsonObj = json.dumps(a)
    print(jsonObj)
    return jsonObj



